@@ -190,11 +190,10 @@ checkout_file_from_input_at (OstreeRepoCheckoutMode mode,
           goto out;
         }
           
-      /* Fall back to filename based setting here due to lack of lsetxattrat */
       if (xattrs)
         {
-          gs_unref_object GFile *path = g_file_get_child (destination_parent, destination_name);
-          if (!gs_file_set_all_xattrs (path, xattrs, cancellable, error))
+          if (!gs_dfd_and_name_set_all_xattrs (destination_dfd, destination_name,
+                                               xattrs, cancellable, error))
             goto out;
         }
     }
@@ -260,8 +259,8 @@ checkout_file_unioning_from_input_at (OstreeRepoCheckoutMode mode,
           
       if (xattrs)
         {
-          gs_unref_object GFile *temp_path = g_file_get_child (destination_parent, temp_filename);
-          if (!gs_file_set_all_xattrs (temp_path, xattrs, cancellable, error))
+          if (!gs_dfd_and_name_set_all_xattrs (destination_dfd, destination_name,
+                                               xattrs, cancellable, error))
             goto out;
         }
     }
@@ -551,9 +550,12 @@ checkout_tree_at (OstreeRepo                        *self,
   gs_unref_variant GVariant *xattrs = NULL;
   gs_unref_object GFileEnumerator *dir_enum = NULL;
 
+  /* Create initially with mode 0700, then chown/chmod only when we're
+   * done.  This avoids anyone else being able to operate on partially
+   * constructed dirs.
+   */
   do
-    res = mkdirat (destination_parent_fd, destination_name,
-                   g_file_info_get_attribute_uint32 (source_info, "unix::mode"));
+    res = mkdirat (destination_parent_fd, destination_name, 0700);
   while (G_UNLIKELY (res == -1 && errno == EINTR));
   if (res == -1)
     {
@@ -566,23 +568,14 @@ checkout_tree_at (OstreeRepo                        *self,
         }
     }
 
-  if (!gs_file_open_dir_fd (destination, &destination_dfd,
-                            cancellable, error))
+  if (!gs_file_open_dir_fd_at (destination_parent_fd, destination_name,
+                               &destination_dfd,
+                               cancellable, error))
     goto out;
 
+  /* Set the xattrs now, so any derived labeling works */
   if (!did_exist && mode != OSTREE_REPO_CHECKOUT_MODE_USER)
     {
-      do
-        res = fchown (destination_dfd,
-                      g_file_info_get_attribute_uint32 (source_info, "unix::uid"),
-                      g_file_info_get_attribute_uint32 (source_info, "unix::gid"));
-      while (G_UNLIKELY (res == -1 && errno == EINTR));
-      if (G_UNLIKELY (res == -1))
-        {
-          ot_util_set_error_from_errno (error, errno);
-          goto out;
-        }
-
       if (!ostree_repo_file_get_xattrs (source, &xattrs, NULL, error))
         goto out;
 
@@ -634,6 +627,33 @@ checkout_tree_at (OstreeRepo                        *self,
         }
     }
 
+  /* We do fchmod/fchown last so that no one else could access the
+   * partially created directory and change content we're laying out.
+   */
+  if (!did_exist && mode != OSTREE_REPO_CHECKOUT_MODE_USER)
+    {
+      do
+        res = fchmod (destination_dfd,
+                      g_file_info_get_attribute_uint32 (source_info, "unix::mode"));
+      while (G_UNLIKELY (res == -1 && errno == EINTR));
+      if (G_UNLIKELY (res == -1))
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+
+      do
+        res = fchown (destination_dfd,
+                      g_file_info_get_attribute_uint32 (source_info, "unix::uid"),
+                      g_file_info_get_attribute_uint32 (source_info, "unix::gid"));
+      while (G_UNLIKELY (res == -1 && errno == EINTR));
+      if (G_UNLIKELY (res == -1))
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+    }
+
   ret = TRUE;
  out:
   if (destination_dfd != -1)
