@@ -62,14 +62,13 @@ out:
 gboolean
 ostree_builtin_log (int           argc,
                     char        **argv,
-                    GFile        *repo_path,
+                    OstreeRepo   *repo,
                     GCancellable *cancellable,
                     GError      **error)
 {
   GOptionContext *context;
   gboolean ret = FALSE;
   const char *rev;
-  gs_unref_object OstreeRepo *repo = NULL;
   gs_free char *checksum = NULL;
   OstreeDumpFlags flags = OSTREE_DUMP_NONE;
 
@@ -79,10 +78,6 @@ ostree_builtin_log (int           argc,
   if (!g_option_context_parse (context, &argc, &argv, error))
     goto out;
 
-  repo = ostree_repo_new (repo_path);
-  if (!ostree_repo_check (repo, error))
-    goto out;
-
   if (opt_raw)
     flags |= OSTREE_DUMP_RAW;
 
