@@ -92,7 +92,7 @@ print_one_file_text (GFile     *f,
   if (opt_checksum)
     {
       if (type == G_FILE_TYPE_DIRECTORY)
-        g_string_append_printf (buf, "%s ", ostree_repo_file_tree_get_content_checksum ((OstreeRepoFile*)f));
+        g_string_append_printf (buf, "%s ", ostree_repo_file_tree_get_contents_checksum ((OstreeRepoFile*)f));
       g_string_append_printf (buf, "%s ", ostree_repo_file_get_checksum ((OstreeRepoFile*)f));
     }
 
@@ -246,8 +246,6 @@ ostree_builtin_ls (int argc, char **argv, OstreeRepo *repo, GCancellable *cancel
   const char *rev;
   int i;
   gs_unref_object GFile *root = NULL;
-  gs_unref_object GFile *f = NULL;
-  gs_unref_object GFileInfo *file_info = NULL;
 
   context = g_option_context_new ("COMMIT PATH [PATH...] - List file paths");
   g_option_context_add_main_entries (context, options, NULL);
@@ -262,7 +260,7 @@ ostree_builtin_ls (int argc, char **argv, OstreeRepo *repo, GCancellable *cancel
     }
   rev = argv[1];
 
-  if (!ostree_repo_read_commit (repo, rev, &root, cancellable, error))
+  if (!ostree_repo_read_commit (repo, rev, &root, NULL, cancellable, error))
     goto out;
 
   if (argc > 2)
